#include "../aoc2022.h"

enum EMoveType {
	eManual,
	eCrateMover9001
};

template <typename Output>
struct SDay05 : public SDay<Output> {
	//Class constructor
	using SDay<Output>::SDay;

	std::vector<std::stack<char>> stacks;

	//Solutions to both days
	Output part1() override final { 
		parseStacks();
		runInstructions(eManual);
		std::string ans = getTops();
		return 0;
	}
	Output part2() override final { 
		parseStacks();
		runInstructions(eCrateMover9001);
		std::string ans = getTops();
		return 0;
	}

	void parseStacks() {
		stacks.clear();
		this->input.clear();
		this->input.seekg(0, std::ios::beg);
		std::stack<std::string> stackStrings;
		int stackCount{ 0 };
		std::string line{ "" };

		while (std::getline(this->input, line))
		{
			if (line == "")
				break; //We have finished collecting stack strings
			stackStrings.push(line);
		}

		//Get the number of stacks needed
		line = stackStrings.top();
		stackCount = line[line.size()-2]-48;
		stackStrings.pop();

		for (int i = 0; i < stackCount; i++)
			stacks.push_back(std::stack<char>());

		int index = 1;
		while (!stackStrings.empty())
		{
			line = stackStrings.top();
			int index = 1;
			for (int i = 0; i < stackCount; i++)
			{
				if (line[index] != ' ')
					stacks[i].push(line[index]);
				index += 4;
			}
			stackStrings.pop();
		}
	}

	void runInstructions(EMoveType moveType) {
		std::string line{ "" };
		
		while (std::getline(this->input, line))
		{
			std::stringstream ss;
			std::vector<int> nums;

			/* Storing the whole string into string stream */
			ss << line;

			/* Running loop till the end of the stream */
			std::string temp;
			int found;
			while (!ss.eof()) {

				/* extracting word by word from stream */
				ss >> temp;

				/* Checking the given word is integer or not */
				if (std::stringstream(temp) >> found)
					nums.push_back(found);

				/* To save from space at the end of string */
				temp = "";
			}

			moveCrates(moveType, nums);
		}
	}

	void moveCrates(EMoveType moveType, const std::vector<int> &nums) {
		//run each step
		std::stack<char> tempStack;
		for (int i = 0; i < nums[0]; i++)
		{
			if (stacks[static_cast<size_t>(nums[1]) - 1].empty())
				break;
			if (moveType == eCrateMover9001)
			{
				tempStack.push(stacks[static_cast<size_t>(nums[1]) - 1].top());
			}
			else 
				stacks[static_cast<size_t>(nums[2]) - 1].push(stacks[static_cast<size_t>(nums[1]) - 1].top());
			
			stacks[static_cast<size_t>(nums[1]) - 1].pop();
		}

		if (moveType == eCrateMover9001)
		{
			while (!tempStack.empty())
			{
				stacks[static_cast<size_t>(nums[2]) - 1].push(tempStack.top());
				tempStack.pop();
			}
		}
	}

	std::string getTops() {
		std::string output;
		for (auto& stack : stacks)
		{
			output += stack.top();
		}
		return output;
	}
};